/*
 * Copyright 1992 Mike Haertel
 * You may use and redistribute this code subject to the GNU Project's
 * Library General Public License.
 */

#include <stddef.h>
#include "bignum.h"

const Suint _bigpow5[] = { SUINT_POW5_TABLE };
const Suint _bigpow10[] = { SUINT_POW10_TABLE };

size_t
_bignbits(size_t nv, Suint v[])
{
	size_t i, nb;
	Suint d;

	i = nv;
	while (i-- && v[i] == 0)
		;
	nb = i * SUINT_BIT;
	for (d = v[i]; d != 0; d >>= 1)
		++nb;
	return nb;
}

size_t
_bigadd(Suint dv[], size_t nv, Suint v[], Suint add)
{
	size_t i;
	Duint c;

	c = add;
	for (i = 0; i < nv; ++i)
		c += v[i], dv[i] = c, c >>= SUINT_BIT;
	if (c != 0)
		dv[i] = c, ++nv;
	return nv;
}

size_t
_bigmuladd(Suint dv[], size_t nv, Suint v[], Suint mul, Suint add)
{
	size_t i;
	Duint c;

	c = add;
	for (i = 0; i < nv; ++i)
		c += (Duint) v[i] * mul, dv[i] = c, c >>= SUINT_BIT;
	if (c != 0)
		dv[i] = c, ++nv;
	return nv;
}

size_t
_bigdivpow2(Suint dv[], size_t nv, Suint v[], unsigned int pow2)
{
	unsigned int skip, shft;
	size_t i;
	Suint lo, hi;

	skip = pow2 / SUINT_BIT;
	shft = pow2 % SUINT_BIT;

	lo = v[skip];
	for (i = skip + 1; i < nv; ++i) {
		hi = v[i];
		dv[i - skip - 1] = lo >> shft | hi << (SUINT_BIT - shft);
		lo = hi;
	}
	if ((dv[i - skip - 1] = lo >> shft) != 0)
		nv = i - skip;
	else
		nv = i - skip - 1;
	if (nv == 0)
		nv = 1;
	return nv;
}

unsigned int
_bigrempow2(size_t nv, Suint v[], unsigned int pow2)
{
	unsigned int rem, skip, shft;

	skip = pow2 / SUINT_BIT;
	shft = pow2 % SUINT_BIT;

	if (shft) {
		if (v[skip] & 1 << (shft - 1))
			rem = 2;
		if (v[skip] & (1 << (shft) - 1) - 1)
			rem |= 1;
	} else {
		--skip;
		if (v[skip] & 1 << (SUINT_BIT - 1))
			rem = 2;
		if (v[skip] & (1 << (SUINT_BIT - 1)) - 1)
			rem |= 1;
	}
	while (skip--)
		if (v[skip])
			rem |= 1;
	return rem;
}
