/*
 * Copyright 1992 Mike Haertel
 * You may use and redistribute this code subject to the GNU Project's
 * Library General Public License.
 */

#include <ctype.h>
#include <float.h>
#include <limits.h>
#include <stddef.h>
#if 0				/* XXX */
#include <stdlib.h>
#endif				/* XXX */
#include "strtod.h"

#define DSIZE (MAX_10_EXP - MIN_10_EXP + 2 * MANT_DIG)
#define FSIZE (-MIN_10_EXP + 2 * MANT_DIG)

#define MAX(A,B) ((A) > (B) ? (A) : (B))

#define ISDIGIT(C) (((unsigned) (C) - '0') < 10)

FLOAT
STRTOF(char *ptr, char **endptr)
{
	char c, *cp, *ep, digs[DSIZE];
	int err, minus, ndig, exp, eminus, eexp;
	FLOAT x;

	err = 1;
	cp = ptr;

	do
		c = *cp++;
	while (isspace(c));

	if (c == '-') {
		minus = 1;
		c = *cp++;
	} else {
		minus = 0;
		if (c == '+')
			c = *cp++;
	}

	if (ISDIGIT(c))
		err = 0;

	while (c == '0')
		c = *cp++;

	ndig = 0;
	while (ISDIGIT(c) && ndig < DSIZE) {
		digs[ndig++] = c - '0';
		c = *cp++;
	}
	exp = ndig;

	if (c == '.') {
		c = *cp++;
		if (ISDIGIT(c))
			err = 0;
		if (ndig == 0)
			while (c == '0') {
				--exp;
				c = *cp++;
			}
		while (ISDIGIT(c) && ndig < DSIZE) {
			digs[ndig++] = c - '0';
			c = *cp++;
		}
	}

	if (err) {
		if (endptr)
			*endptr = ptr;
		return 0.0;
	}

	ep = cp;
	if (c == 'e' || c == 'E') {
		c = *cp++;
		if (c == '-') {
			eminus = 1;
			c = *cp++;
		} else {
			eminus = 0;
			if (c == '+')
				c = *cp++;
		}
		if (ISDIGIT(c)) {
			eexp = 0;
			do {
				if (exp + eexp <= MAX_10_EXP)
					eexp = 10 * eexp + c - '0';
				c = *cp++;
			} while (ISDIGIT(c));
			if (eminus)
				eexp = -eexp;
		} else {
			cp = ep;
			eexp = 0;
		}
	} else
		eexp = 0;

	if (endptr)
		*endptr = cp - 1;

	if (ndig > exp + eexp + FSIZE)
		ndig = MAX(0, exp + eexp + FSIZE);

	if (ndig == 0 || exp + eexp < MIN_10_EXP - MANT_DIG)
		x = 0.0;
	else
		x = _STRTOF(exp + eexp, ndig, digs);
	return minus ? -x : x;

}
/*
 * Copyright 1992 Mike Haertel
 * You may use and redistribute this code subject to the GNU Project's
 * Library General Public License.
 */
